/**
	VersionChecker.js 
	version 1.0

	version 1.1 - added blnSilent parameter - Mark Crossley
	            - made the optional message for URL download only appear if there is a newer version.
	
	version 1.2 - changed Gallery page extract expression
	            - Now checks for web page format changes

	Design & Code ::	Mike Spisak / mspisak@pb.net 

	JavaScript class to facilitate synch'ing your application (widget) 
	with the latest version.  
	
	There are two ways to use this ::
		1 - public hosted :: retrieves a text file on a publicly hosted web site.
			The text file must be formatted in the following way:
				<widget name>
				<version> (this is the current version of your widget )
				<address to get new version>
				<extra message> (optional)
			When a user selects "check for new version.." this class will:
			- retrieve the text file 
			- check to make sure the application names match (if not, throws error)
			- check that the versions match, if they do not
				it will ask to open a web browser to the specified <address to get new version>
			- finally, if there is any <extra message> specified in the text file, it will display that message.
			
			example public method usage:
				var verCheck = new VersionChecker( "http://<your_website>/<your version text file>", "<widget name>", "<running version>" );
				verCheck.checkVersion();
				
				where: 
					<your_website> is the address of your web site hosting your text file
					<your version text file> is the file holding the version information for your widget
						(see formatting above)
					<widget name> the name of your widget, must match the name in your text file
					<running version> the version of the "running" widget
			
		2 - widget gallery hosted :: looks up your widget in the gallery and parses 
			the version number.
			When a user selects "check for new version.." this class will:
			- lookup and parse your widget in the gallery
			- check that the versions match, if they do not
				it will ask to open a web browser to the widget gallery 
			** there are inherent problems with using the widget gallery for version control : 
				-- you need to know your widget ID within the widget gallery in order for 
					this to work.  if this is your widget's first release, then you won't 
					have a widget ID yet! (not until it's posted)
				-- if the widget gallery changes it's HTML format, then the parsing routine
					will most likely break...
	
			example gallery method usage:
				var verCheck = new VersionChecker( "<widget gallery ID>", "<widget name>", "<running version>" );
				verCheck.checkVersion();
					
					where: 
						<widget gallery ID> the ID assigned to my widget in the widget gallery
						<widget name> the name of your widget
						<running version> the version of the "running" widget
						
	Class Usage :: 
	This program is free software; You are granted the right to use 
	and/or redistribute this code under the terms of the GNU General 
	Public License.
	This program is distributed in the hope that it will be useful, 
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. The author
	makes no warranty.
**/

/* 
	VersionChecker.. horray! 
	main javaScript constructor 
*/
function VersionChecker( appURL, appName, appCurrentVersion, blnSilent ) {
	
	this.appURL = appURL;
	this.appName = appName;
	this.appCurrentVersion = appCurrentVersion;
	this.blnSilent = blnSilent;

}

/* 
	checkVersion - method to check the version 
*/
function _VersionChecker_checkVersion() {
	
	// see which method we'll use to check the version..
	if ( isNaN( this.appURL ) ) {  // Public hosted method... 
		
		print("Checking public version...");

		// check for the latest version, retrieve the web file..
		var versionURL = new URL();
		var appLatestVersion = versionURL.fetch( this.appURL );
		
		if ( versionURL.response != "200" ) {
			alert ( "Version Check Error: ouch, there was a problem retrieving data from the host system..." );
		} else {
			// feed the fetched text file into an array...	
			var verArray = new Array();
			verArray = appLatestVersion.split('\n');
			
			// application names don't match... problem!
			if ( this.appName != (verArray[0].replace(/^\s*|\s*$/g,"")) ) {
				alert ("Version Check Error: application version mismatch\nI Expected: " + this.appName + "\nServer Gave Me: " + verArray[0] );
			} else {
				// check versions..
				print("checking widget ver<->web version: " + this.appCurrentVersion + " <-> " + verArray[1].replace(/^\s*|\s*$/g,""));
				if ( this.appCurrentVersion < (verArray[1].replace(/^\s*|\s*$/g,"")) ) {
					var getNewVersion = alert ( "There is a newer version of this Widget (" + this.appName + ") available.\n\nWould you like to open a web browser for downloading?", "Yes please!", "No, not now");
					if ( getNewVersion == 1 ) {
						openURL( (verArray[2].replace(/^\s*|\s*$/g,"")) );
					}
					// is there an extra message you want to show? 
					if ( verArray.length > 3 ) {
						alert ( verArray[3].replace(/^\s*|\s*$/g,"") );
					} 
				} else if (!this.blnSilent) {
					// everything's good!
					alert ( "You are running the latest version of " + this.appName + " : " + this.appCurrentVersion );
				}
			}
		}
	
	} else {	// Gallery method...
		

		// define the parameters...
		//var galleryURL = "http://www.widgetgallery.com/view.php?widget=";
		var galleryURL = "http://widgets.yahoo.com/gallery/view.php?widget=";
		galleryURL += this.appURL;
		var regExp = new RegExp( /"widgetblock_version">Version \d+.\d+ - / );
		print("checking Gallery version, url = " + galleryURL);

		var versionURL = new URL();
		var appLatestVersion = versionURL.fetch( galleryURL );
		
		if ( versionURL.response != "200" ) {
			alert ( "Version Check Error: ouch, there was a problem retrieving data from the widget gallery..." );
		} else {
			var appThisVersion;
			var strRegOut = regExp.exec( appLatestVersion );
			regExp = new RegExp( /\d+.\d+/ );
			appLatestVersion = parseFloat(regExp.exec ( strRegOut ));
			appThisVersion = parseFloat(this.appCurrentVersion);
			print("Version web: " + appLatestVersion + " app: " + appThisVersion);
			if ( appThisVersion < appLatestVersion ) {
				var getNewVersion = alert ("There is a newer version of this Widget (" + this.appName + ") available.\n\nWould you like to open a web browser for downloading?", "Yes please!", "No, not now");
				if ( getNewVersion == 1 ) {
					openURL( galleryURL );
				}
			} else if (!this.blnSilent && (appThisVersion == appLatestVersion)) {
				// everything's good!
				alert ( "You are running the latest version of " + this.appName + " : " + appThisVersion );
			} else if (!this.blnSilent && (appThisVersion > appLatestVersion)) {
				// This app is a later version than the galler!
				alert ( "Eek! You are running a later version of " + this.appName + " (" + appThisVersion + ") than is available in the gallery");
			} else if (appLatestVersion == null) {
				alert ("Version Check Error: Could not extract the version from the widget gallery. The web page format may have changed.");
			}
		}

	} 
		
}


/* build the VersionChecker structure and methods to expose */
VersionChecker.prototype = new Object();
VersionChecker.prototype.checkVersion = _VersionChecker_checkVersion;

